--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity rom_writer is
    Port ( 
           CLK     : in std_logic;
           DI      : in std_logic;
           DO      : out std_logic;
           SEL     : in std_logic;

           DATA    : inout std_logic_vector(7 downto 0) bus;
           ADDR    : out std_logic_vector(20 downto 0);
           RDY     : in std_logic;
           RESET   : out std_logic;
           WE      : out std_logic;
           OE      : out std_logic;

           OSTATE  : out std_logic_vector(2 downto 0);
           OBITS   : out std_logic_vector(2 downto 0);
           XDI1    : in std_logic;
           XCK1    : out std_logic;
           XDO1    : out std_logic;
           XSEL1   : out std_logic;
           XDI2    : in std_logic;
           XCK2    : out std_logic;
           XDO2    : out std_logic;
           XSEL2   : out std_logic
                          );
end rom_writer;

--------------------------------------------------------------------------

architecture behavioral of rom_writer is

        signal cmd   : std_logic_vector(7 downto 0) := "11000000";
        signal ad_3  : std_logic_vector(7 downto 0);
        signal ad_2  : std_logic_vector(7 downto 0);
        signal ad_1  : std_logic_vector(7 downto 0);
        signal bytes : std_logic_vector(7 downto 0);
        signal m_da  : std_logic_vector(7 downto 0);

        signal state : std_logic_vector(2 downto 0) := "000";
        signal nstate : std_logic_vector(2 downto 0) := "000";
        signal bits  : std_logic_vector(2 downto 0) := "000";
begin

    XDO1 <= XDI1;
    XDO2 <= XDI2;
    XSEL1 <= cmd(0);
    XSEL2 <= cmd(1);
    XCK1  <= cmd(2);
    XCK2  <= cmd(3);
    RESET <= cmd(4);

    ADDR(7 downto 0) <= ad_1;
    ADDR(15 downto 8) <= ad_2;
    ADDR(20 downto 16) <= ad_3(4 downto 0);
    OSTATE <= state;
    OBITS <= bits;

    OE <= '0' when (state = "101" and cmd(7 downto 6) = "00" )  else '1';
    WE <= '0' when (state = "110" and cmd(7 downto 6) = "01" 
	 and bits(2 downto 1) = "01")  else '1';

    process (CLK) begin
       if (CLK' event and CLK = '1') then
          state <= nstate;
          if (SEL = '1') then
                state <= "000";
                nstate <= "000";
                cmd(7 downto 6) <= "11";
                DATA <= (others => 'Z');
          else
             if (bits = "111") then
                case nstate is
                   when "000" => nstate <= "001"; -- load cmd
                   when "001" => nstate <= "010"; -- load ad_3
                      if (cmd(7 downto 6) = "11") then
                          nstate <= "000";
                      end if;
                   when "010" => nstate <= "011"; -- load ad_2
                   when "011" => nstate <= "100"; -- load ad_1
                   when "100" => nstate <= "101"; -- load bytes
                   when "101" => nstate <= "110"; -- rw phase 1
                   when "110" => nstate <= "101"; -- rw phase 2
                      if (bytes = "00000001") then
                          nstate <= "000";
                      end if;
                   when others => nstate <= "000"; 
                end case;
             end if;
             if (nstate = "001" and bits = "000") then
                cmd <= ad_3;
             end if;
             DATA <= (others => 'Z');
             DO <= 'Z';
             case nstate is
                when "000" => 
                   case bits is
                      when "000" => DO <= '0';
                      when "001" => DO <= '1';
                      when "010" => DO <= '0';
                      when "011" => DO <= '1';
                      when "100" => DO <= '0';
                      when "101" => DO <= '0';
                      when "110" => DO <= '1';
                      when others => DO <= '1';
                   end case;
                when "001" => 
                   case bits is
                      when "000" => DO <= RDY;
                      when "001" => DO <= '0';
                      when "010" => DO <= XDI1;
                      when "011" => DO <= XDI2;
                      when "100" => DO <= '0';
                      when "101" => DO <= '0';
                      when "110" => DO <= cmd(7);
                      when others => DO <= cmd(6);
                   end case;
                when "110" =>
                   if (cmd(7 downto 6) = "00" ) then
                      DO <= m_da(7);
                   end if;
                   if (cmd(7 downto 6) = "01" ) then
                      DATA <= m_da;
                   end if;
                when others => null;
             end case;
          end if;
       end if;
       if (CLK' event and CLK = '0') then
          if (SEL = '1') then
                bits  <= "000";
          else
             bits <= bits + 1;
             case state is
                when "000" => ad_3  <= ad_3(6 downto 0) & DI; -- load cmd
                when "001" => ad_3  <= ad_3(6 downto 0) & DI; -- load ad_3
                when "010" => ad_2  <= ad_2(6 downto 0) & DI; -- load ad_2
                when "011" => ad_1  <= ad_1(6 downto 0) & DI; -- load ad_1
                when "100" => bytes <= bytes(6 downto 0) & DI; -- load bytes
                when "101" =>
                   if (cmd(7 downto 6) = "01" ) then
                      m_da  <= m_da(6 downto 0) & DI; 
                   end if;
                when "110" =>
                   if (cmd(7 downto 6) = "00" ) then
                      m_da  <= m_da(6 downto 0) & DI; 
                   end if;
                when others => null;
             end case;
             if (bits = "111") then
                if (state = "110") then
                    bytes <= bytes - 1;
                    ad_1 <= ad_1 + 1;
                end if;
                if (state = "101" and cmd(7 downto 6) = "00" ) then
                       m_da <= DATA;
                end if;
             end if;
          end if;
       end if;
    end process;

end behavioral;
