#include <stdio.h>
#include <termios.h>

static struct termios ttysave;
static struct termios ttycur;

int xread(int fd, void *buf, int nbytes) {
	int r;
	int ret = 0;
retry:
	r = read(fd,buf, nbytes);
	if (r == nbytes || r == 0) return ret + nbytes;
	if (r < 0) {
		if (ret == 0) return r;
		return ret;
	}
	buf += r;
	nbytes -= r;
	ret += r;
	goto retry;
}

int fd;

unsigned char ibuf[1024];
unsigned char obuf[1024];

int clk_mode = 1;
int ms1 = 1;
int ms2 = 1;
int ck1 = 1;
int ck2 = 1;
int ckt = 0;

set_mreg() {
  int ret;
  obuf[0] = 0x1c;
  obuf[1] = 0x01;
  obuf[2] = (ckt << 4) | (ck2 << 3) | (ck1 << 2) | (ms2 << 1) | ms1 
            | (clk_mode << 5);
  write (fd, obuf, 3);
  ret = read (fd, ibuf, 3);
  if (ret != 3) printf("set_mreg error\n");
}

do_read_and_print(int addr) {
      int i,ret;

	obuf[0] = 0x00; /* chan = 1 */
	obuf[1] = 5 + 32*2; /* bytes */

	obuf[2] = 0x11;        /* cmd */
	obuf[3] = addr >> 16;  /* ad_3 */
	obuf[4] = addr >> 8;    /* ad_2 */
	obuf[5] = addr;    /* ad_1 */
	obuf[6] = 32;    /* bytes */
        write(fd, obuf, obuf[1] + 2);
printf("write %d bytes\n",obuf[1] + 2);
        ret =xread(fd, ibuf, obuf[1] + 2);
        if (ret !=  obuf[1] + 2) {
	    printf("read error ret = %d expected = %d\n",ret, 5 + 32*2);
            return;
	}
        if (ibuf[2] != 0x53) {
             printf("signature error %02x %02x %02x\n",ibuf[0],ibuf[1],ibuf[2]);
        }
        for (i=0; i<32; i++) {
             if (i==0 || i==16) printf("%06x:",addr+i);
             printf(" %02x",ibuf[i*2 + 8]);
             if (i==15 || i == 31) printf("\n");
        }
}

do_check(int addr) {
      int ret;
      int i=0;
      int man_id;
      int sect_id;

       addr &= 0xff0;
       addr |= 0x04;

	obuf[i++] = 0x00; /* chan = 1 */
	obuf[i++] = 7 * 6; /* bytes */

	/* signature mode */
	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x0a;		/* ad_2 */
	obuf[i++] = 0xaa;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0xaa;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x05;		/* ad_2 */
	obuf[i++] = 0x55;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0x55;		/* data */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x0a;		/* ad_2 */
	obuf[i++] = 0xaa;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0x90;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x11;		/* cmd read */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0;			/* ad_2 */
	obuf[i++] = 0;			/* ad_1 */
	obuf[i++] = 3;			/* bytes */
	obuf[i++] = 0;			/* data (dmy1)*/
        man_id = i;
	obuf[i++] = 0;			/* data (dmy2) */
	obuf[i++] = 0;			/* data (dmy1)*/
	obuf[i++] = 0;			/* data (dmy2) */
	obuf[i++] = 0;			/* data (dmy1)*/
	obuf[i++] = 0;			/* data (dmy2) */

	obuf[i++] = 0x11;		/* cmd read */
	obuf[i++] = addr >> 16;		/* ad_3 */
	obuf[i++] = addr >> 8;		/* ad_2 */
	obuf[i++] = addr;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0;			/* data (dmy1) */
        sect_id = i;
	obuf[i++] = 0;			/* data (dmy2) */

	/* read reset */
	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0;			/* ad_2 */
	obuf[i++] = 0;			/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0xf0;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */


	obuf[1] = i - 2;

        write(fd, obuf, i);
        ret =xread(fd, ibuf, i);
printf("read  %d bytes ret = %d\n", i,ret);
printf("sigcode = %02x %02x %02x\n",ibuf[man_id],ibuf[man_id+2],ibuf[man_id+4]);
printf("sectcode = %02x\n",ibuf[sect_id]);
}

do_write_byte(int addr, int val) {
      int ret;
      int i=0;
	obuf[i++] = 0x00; /* chan = 1 */
	obuf[i++] = 7 * 6; /* bytes */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x0a;		/* ad_2 */
	obuf[i++] = 0xaa;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0xaa;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x05;		/* ad_2 */
	obuf[i++] = 0x55;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0x55;		/* data */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = 0;			/* ad_3 */
	obuf[i++] = 0x0a;		/* ad_2 */
	obuf[i++] = 0xaa;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0xa0;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x51;		/* cmd write */
	obuf[i++] = addr >> 16;		/* ad_3 */
	obuf[i++] = addr >> 8;		/* ad_2 */
	obuf[i++] = addr;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = val;		/* data  */
	obuf[i++] = 0;			/* data (dmy) */

	obuf[i++] = 0x11;		/* cmd read */
	obuf[i++] = addr >> 16;		/* ad_3 */
	obuf[i++] = addr >> 8;		/* ad_2 */
	obuf[i++] = addr;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0;			/* data (dmy1) */
	obuf[i++] = 0;			/* data (dmy2) */

	obuf[i++] = 0x11;		/* cmd read */
	obuf[i++] = addr >> 16;		/* ad_3 */
	obuf[i++] = addr >> 8;		/* ad_2 */
	obuf[i++] = addr;		/* ad_1 */
	obuf[i++] = 1;			/* bytes */
	obuf[i++] = 0;			/* data (dmy1) */
	obuf[i++] = 0;			/* data (dmy2) */

	obuf[1] = i - 2;

        write(fd, obuf, obuf[1] + 2);
printf("write %d bytes (check %d)\n",obuf[1] + 2,i);
        ret =xread(fd, ibuf, obuf[1] + 2);
printf("read  %d bytes \n", ret);
printf("%06x -> %02x -> %02x -> %02x\n",addr,val,ibuf[i - 1 - 7],ibuf[i - 1]);
}

main() {
	int i,ret;
	int cnt = 0;
	int err_cnt = 0;
	speed_t si,so;

	fd = open("/dev/ttyUSB0",2);
	if (fd < 0) {
		perror("open tty");
		exit(1);
	}
	tcgetattr(fd, &ttysave);
        ttysave.c_iflag |= INLCR | ICRNL;
        ttysave.c_lflag |= ICANON | ECHO;
        ttysave.c_oflag |= OPOST | ONLCR;
        ttycur = ttysave;
        ttycur.c_iflag = IGNPAR | IGNBRK;
        ttycur.c_oflag = ONLCR;
        ttycur.c_cflag = CREAD | CS8;
        ttycur.c_lflag &= ~(ICANON | IEXTEN | ECHO | ISIG);
        ttycur.c_cc[VTIME] = 0;
        ttycur.c_cc[VMIN] = 1;
	cfsetispeed(&ttycur, B4000000);
	cfsetospeed(&ttycur, B4000000);
        tcsetattr(fd, TCSANOW, &ttycur);
#if 0
	si = cfgetispeed(&ttycur);
	printf("ispeed = %o \n",si);
	so = cfgetospeed(&ttycur);
	printf("ospeed = %o \n",so);
	exit(1);	

#endif
        ckt = 1;
        ck1 = 0;
        ms1 = 1;
        set_mreg();

        ck1 = 1;
        set_mreg();

        ck1 = 0;
        set_mreg();

        ms1 = 0;
        set_mreg();


	printf("set rst\n");
	obuf[0] = 0x00; /* chan = 1 */
	obuf[1] =  2; /* bytes */

	obuf[2] = 0xd1;    /* cmd */
	obuf[3] = 0xff;    /* ad 3 */

	write (fd, obuf, 4);
	ret = read (fd, ibuf, 32);
	printf("read ret = %d\n",ret);
        printf("O %02x %02x %02x %02x \n"
            ,obuf[0],obuf[1],obuf[2],obuf[3]);
        printf("I %02x %02x %02x %02x \n"
            ,ibuf[0],ibuf[1],ibuf[2],ibuf[3]);

/* read data */
	printf("read data\n");
	obuf[0] = 0x00; /* chan = 1 */
	obuf[1] = 11; /* bytes */

	obuf[2] = 0x11;    /* cmd */
	obuf[3] = 0x00;    /* ad_3 */
	obuf[4] = 0x00;    /* ad_2 */
	obuf[5] = 0x04;    /* ad_1 */
	obuf[6] = 0x03;    /* bytes */
	obuf[7] = 0xff;    /* dmy 1 */
	obuf[8] = 0xff;    /* dmy 2 */
	obuf[9] = 0xff;    /* dmy 1 */
	obuf[10] = 0xff;    /* dmy 2 */
	obuf[11] = 0xff;    /* dmy 1 */
	obuf[12] = 0xff;    /* dmy 2 */
	write (fd, obuf, 13);
	ret = read (fd, ibuf, 32);
	printf("read ret = %d\n",ret);
        printf("O %02x %02x %02x %02x %02x %02x %02x"
            ,obuf[0],obuf[1],obuf[2],obuf[3],obuf[4],obuf[5],obuf[6]);
        printf("  %02x %02x %02x %02x %02x %02x \n"
            ,obuf[7],obuf[8],obuf[9],obuf[10],obuf[11],obuf[12]);
        printf("I %02x %02x %02x %02x %02x %02x %02x"
            ,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5],ibuf[6]);
        printf("  %02x %02x %02x %02x %02x %02x \n"
            ,ibuf[7],ibuf[8],ibuf[9],ibuf[10],ibuf[11],ibuf[12]);

       do_read_and_print(0);
       do_read_and_print(0x100);
       do_check(0x100);
       do_write_byte(0x100, 0x5a);
}
